 /**
 * Title: Advanced02 - Examples about Properties and System Properties
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.io.*;
import java.util.* ;


/** Application showing the usage of Properties and System Properties.	*/
public class Advanced02 {


	/** The main method launched at start-up. 		*/
	public static void main(String args[]) {

		Advanced02 application = new Advanced02() ;
		String fp = File.separator ;
		String path = "adriani" + fp + "tutorial" + fp + "advanced" ;

		try {
			application.loadConfiguration(path + fp + "test.cfg") ;
			application.showDefaults() ;
		}
		catch(IOException e) { System.err.println("*IOError reading properties ...") ; }
		catch(Exception e)   { System.err.println("*General error reading properties ...") ; }
	}


	/** Load some properties from the <CODE>test.cfg</CODE> file.		*/
	public void loadConfiguration(String path) throws FileNotFoundException, IOException {

		InputStream file = null ;
		Properties properties = new Properties() ;
		file = (InputStream) new FileInputStream(path) ;
		properties.load(file) ;

		if (properties != null)  {

	    	String course 	= properties.getProperty("Course") ;
	    	String lecture	= properties.getProperty("Lecture") ;
	    	String example	= properties.getProperty("Example") ;
	    	System.out.println("\n*** APPLICATION PROPERTIES :\n") ;
	    	System.out.println(course + "/" + lecture + "/" + example) ;
	    	properties.put("Opened", "true");
		}
		file.close( );

		FileOutputStream output = new FileOutputStream(path + ".new");
		// Second argument will be printed in the file's header
		properties.store(output, path + ": " + new Date( ));
		output.close( );
	}


	/** Prints on the console the default System Properties.			*/
	public void showDefaults() throws FileNotFoundException, IOException {

        Properties system = System.getProperties();
        Enumeration names = system.propertyNames();
        System.out.println("\n*** DEFAULT SYSTEM PROPERTIES :\n") ;

        String key;
        while (names.hasMoreElements()) {

            key = (String) names.nextElement();
            String value = system.getProperty(key) ;
            print(key + " = ", 38);
            if (value.length() > 0) System.out.println(value) ;
            else System.out.println("[no set]") ;
        }
	}

	/** Print some blank spaces at the end of each word in order to format the output.*/
	public void print(String line, int space) {

		System.out.print(line) ;
		int size = space - line.length() ;
		for ( int j = 0; j < size; j++) System.out.print(" ") ;
	}

}	// End of class
