 /**
 * Title: Advanced04 - Practising with threads basic concepts.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.io.*;
import java.util.* ;

/** Class testing main thread features.
 *	Each instance of Adavanced04 implements a single Thread.	*/
 public class Advanced04 extends Thread	{

	/**	Main method where all threads are created. 				*/
    public static void main(String args[])    {

		int i ;
		int n = -1 ;
		while ( n != 0) {	// If (n == 0) we quit only the main thread !!!

			Thread[] list ;
			Advanced04.printMenu() ;
			n = Advanced04.getNumber() ;

			switch (n) {

			// Creating a new thread object
			case 1:	System.out.print("Enter thread number: ") ;
					Advanced04 tmp = new Advanced04( getNumber() ) ;
					tmp.start() ;
					break ;

			// Stopping an existing thread by ending his run method.
			case 2:	System.out.print("Enter thread number: ") ;
					int k = getNumber() ;
					list = Advanced04.getList();
					for (i = 0; i < list.length; i++) if (list[i] != null)
						if ( (list[i].getName()).equalsIgnoreCase("T0" + k) ) {

							System.out.println("Killing thread " + list[i].getName()) ;
							// The Thread class hasn't the "running" field --> casting
							( (Advanced04) list[i] ).running = false ;
						}
					break ;

			// Listing all threads in this group
			// Hint: try to quit application with some threads still alive !
			case 3: list = Advanced04.getList() ;
					System.out.println("Name    Alive   Daemon   Priority") ;
					for (i = 0; i < list.length; i++) if (list[i] != null) {

							System.out.print(list[i].getName() + "    ") ;
							System.out.print(list[i].isAlive() + "    ") ;
							System.out.print(list[i].isDaemon() + "    ") ;
							System.out.println(list[i].getPriority()) ;
					}
					System.out.println() ;
					break ;

			// Enter continuos monitor state for 0.1 seconds
			case 4:	Advanced04.monitor = true ;
					try {	sleep(100) ; }
					catch(InterruptedException e) { System.out.println("Sleep interrupted ..."); }
					Advanced04.monitor = false ;
					break ;

			} // End of switch

		} // End of while

    }

// ****************************			ACTUAL Advanced04 METHODS		***************************************+

	// CLASS ATTRIBUTES
	long timer ;
	boolean running ;
	static boolean monitor ;

	/** Default constructor. We need to define this since we define another constructor.	*/
	public Advanced04()    {

		super() ;			// Calls the Thread() constructor
		this.timer = 0 ;	// Counts milliseconds from birth time
	}

	/** Stantard constructor. Each thread will have name "T010", "T011", "T012" etc. 	*/
    public Advanced04(int index)    {

		this() ;
		this.setName("T0" + index) ;
		System.out.println("New thread created!") ;
    }

	/** Called when this thread is started. It just counts the number of seconds it is alive.		 */
    public void run()    {

		running = true ;
		while (running) {

			timer++ ;
			try {	sleep(1) ; }
			catch(InterruptedException e) { System.out.println("Sleep interrupted ..."); }
			if (monitor) System.out.println(this.getName() + " is running: " + Thread.currentThread() ) ;
		}

		// Notice: this line will be executed concurrently with the 'main while-switch loop',
		// this is an example of "asynchronous behaviour"  ... just try it !
		System.out.println("Thread stopped after " + timer + " milliseconds ...") ;
    }

// ****************************			STATIC UTIL METHODS			***************************************+

    /** Prints the main menu.								 */
    public static void printMenu()    {

		System.out.println() ;
		System.out.println("1 - Start a new thread") ;
		System.out.println("2 - Kill an existing thread") ;
		System.out.println("3 - List all threads") ;
		System.out.println("4 - Enter 'continuos monitor' for just a moment") ;
		System.out.println("0 - Exit") ;
		System.out.println() ;
		System.out.print("Enter your choice: ") ;
	}

	/** Returns a numeric value from standard input. 			*/
    public static int getNumber()    {

		int n = 0 ;
		int output = 99 ;		// Default value if an exception occurs
		String input = "" ;
        try {
			while (n != 13) {
				n = System.in.read() ;
				if (n != 13) input += (char) n ;
			}
			System.in.skip(1) ;
			output = Integer.parseInt(input) ;
		} catch(Exception e) { System.err.println("Wrong input: type only numbers, please ..."); }

		System.out.println() ;
		return output ;
	}

	/** Returns the array of active threads in this group.		*/
    public static Thread[] getList()    {

		ThreadGroup group = Thread.currentThread().getThreadGroup();
		Thread[] list = new Thread[group.activeCount()];
		if (group != null) group.enumerate(list) ;
		//group.list() ;

		return list ;
	}

}	// End of class
