 /**
 * Title: Advanced05 - Practising with the thread-runnable concept.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

/** Class testing main runnable features.<P>
*
* <B>Remark:</B> This class calls the method <CODE>join()</CODE> in order to show how it works.
* This is achieved by triggering the class's field "target": the Advanced06 class modifies this field,
* setting it to a not-null value, by this mechanism the <CODE>join()</CODE> method is executed from THIS thread!<P>
*
* It would not be possible to call anhoter method of this class (from Advanced06) to perform this operation,
* because in such a case the "paused" thread would have been the Advanced06 thread, i.e. the main thread!<BR>
* See the <CODE>test()</CODE> method below to understand this concept.
*/
public class Advanced05 implements Runnable	{

	// CLASS ATTRIBUTES
	long timer ;					// Number of milliseconds this thread is alive
	Thread theThread ;				// Thread of this runnable object
	Thread target ;					// Target thread for the join() method
	boolean running ;				// Main flag for this thread
	static boolean monitor ;		// Common state for all Advanced05 instances

	/** Stantard constructor for this class.			*/
    public Advanced05()    {

		super() ;
		this.timer = 0 ;
		System.out.println("New runnable created!") ;
    }

	/** Called when this thread is started. It just counts the number of seconds it is alive
     *  and contains the join() call to another target thread.								*/
    public void run()    {

		running = true ;

		while (running) {

			timer++ ;
			try {	theThread.sleep(1) ; }
			catch(InterruptedException e) { System.out.println("Sleep interrupted ..."); }
			if (monitor) System.out.println(theThread.getName() + " is running: " + Thread.currentThread() ) ;

			// The call to join() MUST be included into *this* thread (see test() method below)
			if ( (target != null) && (theThread == Thread.currentThread()) ) {

				System.out.println("Joining " + target + " from " + this) ;
				try 							{ target.join(2000) ;							}
				catch(InterruptedException e) 	{ System.out.println("Join interrupted ..."); 	}
				target = null ;
			}
		}
		System.out.println("Runnable stopped after " + timer + " milliseconds ...") ;
    }

    /** Called by Advanced06 to explain how threads can be "mixed"	during execution. */
    public void test(Thread thread)    {

		System.out.println(theThread.getName() + " code exectuted by: " + Thread.currentThread() ) ;

		// Following line will NEVER be executed !!!
		// This explain why we included join() inside the main loop of this class (see above).
		if (theThread == Thread.currentThread()) System.out.println("THEY'RE THE SAME THREAD !") ;
		else {
			System.out.println("THEY'RE DIFFERENT THREADS !") ;
			System.out.println("Trying to join " + thread + " from " + this) ;
			try 							{ thread.join(2000) ;							}
			catch(InterruptedException e) 	{ System.out.println("Join interrupted ..."); 	}
		}
	}

}	// End of class
