 /**
 * Title: Advanced06 - Practising with threads basic concepts.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.io.*;
import java.util.* ;

/** Class testing main thread features.		 	*/
public class Advanced06 extends Advanced04	{

	// CLASS ATTRIBUTES
	Thread[] list ;				// List of threads in current ThreadGroup
	Hashtable threads ;			// Hashtable containing the Runnable instances
	int maxIndex ;				// The biggest index entered by the user

	/**	Main method: just creates this class and runs the loopMenu() method. */
    public static void main(String args[])    {

		Advanced06 theApp = new Advanced06() ;
		theApp.loopMenu() ;
	}

	/**	Main method where threads are handled. 			*/
    public void loopMenu()    {

		int i ;
		int n = -1 ;
		while ( n != 0) {

			this.printMenu() ;
			n = this.getNumber() ;		// Inherited from Advanced04

			switch (n) {

			case 1:	startNewOne() ;		break ;

			case 2: stopOne() ;			break ;

			case 3: long timer = 0 ;
					System.out.println("Name    Priority  Time") ;
					// Using a very rough system to list all threads: see maxIndex below
					for (i = 0; i <= maxIndex; i++) {

						Thread thread = null;
						Thread 		tmp1 = findThread(i) ;
						Advanced05 	tmp2 = findRunnable(i) ;
						if (tmp1 != null) thread = tmp1 ;
							else if (tmp2 != null) thread = tmp2.theThread ;

						if (thread != null) {

							System.out.print(thread.getName() + "    ") ;
							System.out.print(thread.getPriority() + "         ") ;
							if (thread instanceof Advanced04) timer = ((Advanced04)thread).timer ;
								else timer = tmp2.timer ;
							System.out.println(timer + " ms") ;
						}
					}
					System.out.println() ;
					break ;

			case 4:	Advanced04.monitor = true ;
					Advanced05.monitor = true ;
					// Calling yield() on *this* thread is similar to a short sleep() !
					Thread.currentThread().yield() ;
					Advanced04.monitor = false ;
					Advanced05.monitor = false ;
					break ;

			case 5: joinOne();			break ;

			// Not requested, but it helps to understand why join() can not be called from here!
			case 6:	System.out.print("Enter runnable number: ") ;
					int index = this.getNumber() ;
					System.out.print("Enter thread number: ") ;
					int second = this.getNumber() ;
					Advanced05 choosen = findRunnable(index) ;
					Thread target = findThread(second) ;
					choosen.test(target) ;
					break ;

			} // End of switch

		} // End of while

    }

	/**	Here new threads (of both types) are created. 					*/
    public void startNewOne()    {

		if (threads == null) threads = new Hashtable() ;
		System.out.print("Enter thread or runnable number: ") ;
		int index = getNumber() ;

		// Case 1: creating a Thread like in Advanced04
		if (index > 9) {

			Advanced04 tmp1 = new Advanced04(index) ;
			tmp1.start() ;

		// Case 2: creating a Runnable like in Advanced05
		// Remark: the Hashtable is used only for Runnable objects.
		} else if (index > 0) {

			Advanced05 tmp2 = new Advanced05() ;
			threads.put(new Integer(index), tmp2) ;
			Thread newThread = new Thread(tmp2) ;
			tmp2.theThread = newThread ;
			newThread.setName("R00" + index) ;
			newThread.setPriority(index) ;
			newThread.start() ;
		}

		// Updating the maxIndex value (see above, option "3")
		if (index > maxIndex) maxIndex = index ;
	}

	/**	Here existing threads are destroyed.			*/
    public void stopOne()    {

		System.out.print("Enter thread or runnable number: ") ;
		int index = this.getNumber() ;
		Thread normal = findThread(index) ;

		if (normal != null) {
			// Case 1: destroying a Thread as defined in Advanced04
			System.out.println("Killing thread " + normal.getName()) ;
			( (Advanced04) normal ).running = false ;
		}
		else {
			// Case 2: destroying a Runnable as defined in Advanced05
			Advanced05 runnable = findRunnable(index) ;
			if (runnable != null) {
				System.out.println("Killing runnable " + runnable.theThread.getName()) ;
				runnable.running = false ;
			}
		}
	}

	/**	Tell to a given thread (if is a Runnable type) to join anohter thread.	*/
    public void joinOne()    {

		System.out.print("Enter number of runnable that will call join(): ") ;
		int first = this.getNumber() ;
		System.out.print("Enter number of thread that will gain CPU during to the join(): ") ;
		int second = this.getNumber() ;

		Advanced05 choosen = findRunnable(first) ;
		Thread target = findThread(second) ;
		choosen.target = target ;			// see Advanced05 source code
	}

	/**	Return a Thread object as defined in Advanced04, given the index.	*/
    public Thread findThread(int index)    {

		list = this.getList();		// Inherited from Advanced04
		for (int i = 0; i < list.length; i++) if (list[i] != null)
			if ( (list[i].getName()).equalsIgnoreCase("T0" + index) ) return list[i] ;

		return null ;
	}

	/**	Return a Runnable object as defined in Advanced05, given the index.
	*   Remark: the runnable is obtained by quering the Hashtable "threads".*/
    public Advanced05 findRunnable(int index)    {

		list = this.getList();		// Inherited from Advanced04
		for (int i = 0; i < list.length; i++) if (list[i] != null)
			if ( (list[i].getName()).equalsIgnoreCase("R00" + index) )
				return (Advanced05) threads.get( new Integer(index) ) ;

		return null ;
	}

	/** Prints the main menu. Overrides the method defined in Advanced01	 */
    public static void printMenu()    {

		System.out.println() ;
		System.out.println("1 - Start a new thread/runnable") ;
		System.out.println("2 - Kill an existing thread/runnable") ;
		System.out.println("3 - List all threads/runnables") ;
		System.out.println("4 - Enter 'continuos monitor' for just a moment") ;
		System.out.println("5 - Join an existing thread from a runnable") ;
		System.out.println("6 - Invoke a runnable from the main thread (this)") ;
		System.out.println("0 - Exit") ;
		System.out.println() ;
		System.out.print("Enter your choice: ") ;
	}

}	// End of class
