 /**
 * Title: Advanced07 - How race conditions can be avoided.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.io.*;
import java.util.*;

/** Class testing some thread features about synchronisation. 	*/
public class Advanced07 {

	// CLASS ATTRIBUTES
	Random dice ;
    int totalMeters  = 0;
    boolean goodExample = false ;
    //boolean goodExample = true ;

	/**	Main method: just creates this class by running the constructor. */
    public static void main(String args[]) {

        Advanced07 theApp = new Advanced07();
    }

	/**	Standard constructor. 	*/
    public Advanced07()    {

        this.dice = new Random() ;
        // Creating two helper classes (see below)
        CounterThread first = new CounterThread(this);
        CounterThread second = new CounterThread(this);
        first.start();
        second.start();

        while ( first.isAlive() || second.isAlive() ) {
			// This thread runs longer than other two threads
            try 							{ Thread.sleep( 1000 ); }
            catch (InterruptedException e) 	{ /* do nothing */ 		}
        }

        System.out.println("Total counter is: " + this.totalMeters);
    }

    /**	This is the WRONG method, executed by the two threads in the same time. */
    public void fooUpdateTotal() {

		int oldValue = this.totalMeters ;
		try 							{ Thread.sleep(2); }
		catch (InterruptedException e) 	{ /* do nothing */ }
        this.totalMeters = oldValue + 1;
	}

	/**	This is the RIGHT method, executed only by one thread at time. */
    public synchronized void updateTotal()	{

		int oldValue = this.totalMeters ;
		try 							{ Thread.sleep(2); }
		catch (InterruptedException e) 	{ /* do nothing */ }
        this.totalMeters = oldValue + 1;
	}

}	// End of class


// ************************		HELPER CLASS	*******************************


/**	Helper class represeting one single thread. */
class CounterThread extends Thread {

    Advanced07	theApp;
    int         threadMeters = 0;

	/**	Constructor for the helper class. */
    public CounterThread(Advanced07 above)    {

        theApp = above;
    }

	/**	The method running when the thread is started. */
    public void run()    {

        System.out.println(currentThread() + " started.");

        for (int i = 0; i < 250; i++) {

              threadMeters++;
              if (theApp.goodExample) theApp.updateTotal();
              	else theApp.fooUpdateTotal();

              // We force the two thread to run asynchronously by introducing random sleep
              long time = (long) (5*(theApp.dice.nextDouble()) + 1) ;
              try 								{ Thread.sleep(time); 	}
		      catch (InterruptedException e) 	{ /* do nothing */ 		}
        }

        System.out.println(currentThread() + " counted " + threadMeters);
    }

}	// End of helper class
