 /**
 * Title: Advanced08 - Practising with the object's monitor bit.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.util.*;

/** Class showing about to handle the object's monitor. 	*/
public class Advanced08 {

	// CLASS ATTRIBUTE
	int timer ;						// The clock timer
	boolean running, debug ;		// The debug field is here to teach "good coding"
	String reference ;				// The object which monitor will be used

	/**	Main method: just create this class by calling the constructor. */
    public static void main(String args[])    {

        System.out.println();
        Advanced08 theApp = new Advanced08();
    }

	/**	Standard constructor. 	*/
    public Advanced08()    {

		this.debug = true ;
		this.reference = "" ;
        SingleThread first  = new SingleThread(this, 0);		// Mode 0
        SingleThread second = new SingleThread(this, 1);		// Mode 1
        first.start();
        second.start();
        startTimer() ;
   }

    /**	The method containing the <CODE>synchronized</CODE> block code. */
    public void block(int type)    {

		synchronized (reference) {

			// Step 1: we get into and wait 0.5 second
			if (debug) System.out.println(getSignature() + "into block") ;
			while (timer < 500) ;	// Do nothing

			// Step 2: thread "0" will call wait() to release reference's monitor
			if (type == 0 ) {

				if (debug) System.out.println(getSignature() + "waiting()") ;
				try { reference.wait(); }
	    		catch (InterruptedException e) { /* do nothing */ }
			}

			// Step 3: thread "0" will reach this line only after a "reference.notify()" event
	    	if (debug) System.out.println(getSignature() + "out from block") ;
		}
	}

	/**	This is an "empty" synchronized method proving indipendence from "reference". */
    public synchronized void fooMethod(int sleepTime) {

		if (debug) System.out.println(getSignature() + "into foo()") ;
		try 							{ Thread.sleep(sleepTime); 	}
	    catch (InterruptedException e) 	{ /* do nothing */ 			}
		if (debug) System.out.println(getSignature() + "out from foo()") ;
	}

	/**	This is the "rough timestamp" used to understand the time line.<BR>
	* It's not accurate because 1ms is comparable with time of code execution ...
	*/
    public void startTimer() {

		timer = 0 ;
		running = true ;
		while (running) {
			timer++ ;
			try { Thread.sleep(1); }
	    	catch (InterruptedException e) { /* do nothing */ }
		}
	}

    /**	This method just return the name of current Thread and a "timestamp". */
    public String getSignature()    {

		String name = Thread.currentThread().getName() ;
		name += " (" + timer + "): ";
		return name ;
	}

} // End of class


// **********************		HELPER CLASS		****************************


/**	Helper class representing one single thread. */
class SingleThread extends Thread {

    int mode ;					// Can be 0 (mode-0) or 1 (mode-1)
    Advanced08	theApp;

	/**	Constructor for the helper class. */
    public SingleThread(Advanced08 above, int modeType)    {

        this.theApp = above;
        this.mode = modeType ;
    }

	/**	The method running when the thread is started. */
    public void run()    {

		System.out.println(theApp.getSignature() + "started in mode " + mode);

        if (mode == 0) {

			// Mode 0: we run only block()
			System.out.print(theApp.getSignature() + "entering block ---> ");
			theApp.block(mode) ;

		} else {

			// Mode 1: we run quickly foo(), then we try block()
			System.out.print(theApp.getSignature() + "entering foo() ---> ");
			theApp.fooMethod(500);
			System.out.print(theApp.getSignature() + "entering block ---> ");
			theApp.block(mode) ;

			// If we got here, we assume thread "0" is still waiting
			synchronized (theApp.reference) {

				System.out.println(theApp.getSignature() + "notify()") ;
				theApp.reference.notify() ;
				try 							{ Thread.sleep(500); 	}
		    	catch (InterruptedException e) 	{ /* do nothing */ 		}
		    	System.out.println(theApp.getSignature() + "out from notify()") ;
			}
			// Only NOW the reference's monitor will be released (end of code block)

			// Now all threads are ended --> we close the main one.
			System.out.println();
			theApp.running = false ;
		}
    }


} // End of helper class