 /**
 * Title: Advanced09 - An example of animated applet.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;

/** Class showing how to use threads to create animation. 				*/
public class Advanced09 extends Applet implements Runnable, MouseListener {

	// ATTRIBUTES
	int radius ;
    int figures = 5 ;
    int clock = 100 ;
    int imageID = 42 ;	// Used by the MediaTracker (can be any "unique" number)

    Advanced10 glass ;
    Advanced10[] pictures = new Advanced10[figures + 1] ;

	Thread theThread ;
	boolean	running, hasImage ;

	MediaTracker 	theTracker ;
	Graphics  		theBufferGraph ;
	Image     		theBuffer, image ;
    Dimension 		bufferSize ;
    Color 			foreground, background ;

	/** Initializes the applet.		*/
    public void init() {

		this.running = true ;

		// Retrieving parameters
		try 				{ 	this.radius	= getIntegerParameter("radius") ;		}
		catch(Exception e) 	{	this.radius = 30 ;									}
		try 				{ 	this.foreground= getColorParameter("foreground") ;	}
		catch(Exception e) 	{	this.foreground= Color.blue ;						}
		try 				{ 	this.background= getColorParameter("background") ;	}
		catch(Exception e) 	{	this.background= Color.blue ;						}

		// Creating the glass and the pictures
		glass = new Advanced10(getSize().width, getSize().height) ;
		glass.setParams(Color.CYAN, radius, null) ;
		Image large = getImage(getCodeBase(), "test.gif");
		image = large.getScaledInstance(radius, radius, Image.SCALE_DEFAULT) ;
        theTracker = new MediaTracker(this);
        theTracker.addImage(image, imageID);

        for (int i = 1; i <= figures; i++) {
			pictures[i] = new Advanced10(glass, getSize().width, getSize().height);
            pictures[i].setParams(foreground, radius, "Icon" + i) ;
            pictures[i].setParent(this, i) ;
		}

        addMouseListener(this);
        setForeground(foreground);
        setBackground(background);
    }

	/** Starts the main thread. */
    public void start() {

        if (theThread == null) theThread = new Thread(this);
        theThread.start( );
    }

	/** Stops the main thread.	*/
    public void stop() {

        if (theThread != null && theThread.isAlive( )) this.running = false ;
    }

	/** Contains the main loop which runs all time. 	*/
    public void run() {

        while (running) {

			glass.updatePosition(null, getSize().width, getSize().height);
            for (int i = 1; i <= figures; i++) pictures[i].updatePosition(
				glass.getPosition(), getSize().width, getSize().height);

        	repaint();
      		try 							{	Thread.sleep(clock); 	}
            catch (InterruptedException e) 	{ 	/* Do nothing */ 		}
        }
    }

	/** Change the glass position in the location choosen by the user.	*/
    public void mouseClicked (MouseEvent theEvent) {

        glass.moveTo(theEvent.getX(), theEvent.getY());
    }

	/** Draws all images on the Applet surface.	*/
    public void paint(Graphics g) {

		// 1. Cleaning the surface
        g.setColor(background);
        g.fillRect(0, 0, getSize().width, getSize().height);

		// 2. If the image has been loaded, we draw it
        if ( theTracker.checkID(imageID, true) ) {
			for (int i = 1; i <= figures; i++) {
				int x = pictures[i].x - radius ;
				int y = pictures[i].y - radius ;
        		hasImage = g.drawImage(image, x, y, null) ;
			}
        } else repaint() ;

		// 3. Otherwise we paint just simple circles (and the glass)
        if (! hasImage) for (int i = 1; i <= figures; i++) pictures[i].paint(g);
        glass.paint(g);
    }

	/** Overriding the standard update() method, using the double buffering technique.	*/
    public final synchronized void update(Graphics g) {

		boolean changeX = false ;
		boolean changeY = false ;

		if (bufferSize != null) {

			changeX = ( bufferSize.width  != getSize().width  ) ;
			changeY = ( bufferSize.height != getSize().height ) ;

		} else if (bufferSize == null || changeX || changeY) {

            bufferSize = getSize();
            theBuffer = createImage(bufferSize.width, bufferSize.height);
            theBufferGraph = theBuffer.getGraphics( );
        }

        paint(theBufferGraph);
        g.drawImage(theBuffer, 0, 0, null);
    }

	/** Return the array with all the pictures.			*/
    public Advanced10[] getPictures() 	{	return pictures ;	}


// ********************************		METHODS RETRIEVING PARAMETERS	*****************************


	/** Parse an <CODE>integer</CODE> parameter retrieved from the <CODE>getParameter()</CODE>
	*   applet's paramter (HTML tag).																*/
	public int getIntegerParameter(String parameter) {

		int result = 0 ;
		try {
			String tmp = getParameter(parameter).trim() ;
			result = Integer.parseInt(tmp) ;
		} catch(Exception e) 	{	result = 0;}

		return result ;
	}

	/** Parse a <CODE>boolean</CODE> parameter retrieved from the <CODE>getParameter()</CODE>
	*   applet's paramter (HTML tag).																*/
	public boolean getBooleanParameter(String parameter) {

		boolean result = false ;
		try {
			String tmp = getParameter(parameter).trim() ;
			Boolean value = Boolean.valueOf(tmp) ;
			result = value.booleanValue() ;
		} catch(Exception e) 	{	result = false ;}

		return result ;
	}

	/** Parse a <CODE>RGB</CODE> color parameter retrieved from the <CODE>getParameter()</CODE>
	 *	applet's paramter (HTML tag).																*/
	public Color getColorParameter(String parameter) {

		// Default values
		String tmp = null ;
		int red = 0 ;		int green = 0 ;			int blue = 255 ;

		try {
			tmp = getParameter(parameter).trim() ;
			String r = tmp.substring(0,2) ;
			String g = tmp.substring(2,4) ;
			String b = tmp.substring(4,6) ;
			red 	= Integer.parseInt(r,16) ;
			green 	= Integer.parseInt(g,16) ;
			blue 	= Integer.parseInt(b,16) ;
		} catch(Exception e) {	System.err.println("Error parsing color: " + tmp) ;	}

		return new Color(red, green, blue) ;
	}

// **************************		METHODS NOT USED	*****************************

    public void mouseEntered (MouseEvent theEvent) { }
    public void mouseExited  (MouseEvent theEvent) { }
    public void mousePressed (MouseEvent theEvent) { }
    public void mouseReleased(MouseEvent theEvent) { }

}	// End of class