 /**
 * Title: Advanced10 - Class representing the images used by the Advanced09 class.
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.util.Observer;
import java.util.Observable;


/** The class representing a circle/picture, or the glass.	*/
class Advanced10 extends Observable implements Observer {

	Color color ;		// The color (only if we draw the circle)
	Point focus ;		// The glass coordinates (if we have the glass focus)
	String text ;		// The text written on the circle ("icon1")

	Advanced09 parent ;
	boolean isGlass = false ;

	int radius ;
    int MaxAcc = 2 ;
    int MaxVel = 4 ;
    int index, target ;		// The reference number for this picture

	int x, y ;
    int velX, velY ;

    /** Used to create the glass object.					*/
	public Advanced10(int width, int height) {

		this(null, width, height) ;
		this.isGlass = true ;
	}

	/** Used to create a standard circle/picture.			*/
    public Advanced10(Advanced10 glass, int width, int height) {

        x = (int) (width * Math.random());
        y = (int) (height * Math.random());
		if (glass != null) glass.addObserver(this);
    }

	/** Used to set graphic parameters for this pictures.	*/
    public void setParams(Color c, int r, String s) {

		this.color = c ;
		this.radius = r ;
		this.text = s ;
	}

	/** Used to pass a reference to the parent applet.	*/
    public void setParent(Advanced09 parent, int index) {

		this.parent = parent ;
		this.index = index ;
	}

	/** Paint this picture (if "circle" just fills an Arc).	*/
    public void paint(Graphics g) {

		// Draw the circle only if not below the glass
		if (focus == null) {
			g.setColor(color);
			g.fillArc( (int) (x - radius), (int) (y - radius),
				(int) radius, (int) radius, 0, 360);
			g.setColor(Color.BLACK) ;
			if (text != null) g.drawString(text, (10 + x - radius), (y - radius/2)) ;
		}

		// Drawing the glass
		if (isGlass) {
			g.setColor(Color.BLACK) ;
			g.drawOval( (int) (1 + x - radius), (int) (1 + y - radius),
				(int) radius, (int) radius);
			g.fillRect( (int)(x - radius/2 - 4), y, 8, radius);
			if (target != 0) g.drawString("" + target, (x - radius/2), (y - radius/2)) ;
		}
    }

	/** Update the x,y and velocities considering current glass location.	*/
    public void updatePosition(Point glassPoint, int w, int h) {

		// If not below the glass: random movement
        velX = (int) (velX + 2.0*MaxAcc*Math.random() - MaxAcc);
        velX = (velX > MaxVel)?MaxVel:velX;
        velY = (int) (velY + 2.0*MaxAcc*Math.random() - MaxAcc);
        velY = (velY > MaxVel)?MaxVel:velY;

		// If it's not the glass itself we move FAR from the glass
		if (glassPoint != null) {
	        int dx = x - glassPoint.x ;
	        int dy = y - glassPoint.y ;
			if (dx != 0) velX += (int) (Math.abs(dx))/dx ;
			if (dy != 0) velY += (int) (Math.abs(dy))/dy ;
		}

		// Updating the position (following the glass if we have the focus)
		if (focus == null) {
			x = x + velX ;
			y = y + velY ;
		} else {
	        x = glassPoint.x ;
	        y = glassPoint.y ;
		}

		// Bouncing at the borders
        if (x < radius) {
            x = radius;
            velX = (int) (2.0 * Math.abs(velX));
        }
        if (y < radius) {
            y = radius;
            velY = (int) (2.0 * Math.abs(velY));
        }
	  	if (x > w - radius) {
		    x = w - radius;
		    velX = (int) (-2.0 * Math.abs(velX));
		}
		if (y > h - radius) {
		    y = h - radius;
		    velY = (int) (-2.0 * Math.abs(velY));
		}

        // Bouncing from others pictures
        Advanced10[] others = null ;
        if (parent != null) others = parent.getPictures() ;
        if (others != null) for (int i = 1; i < others.length ; i++) {
			int otherX = others[i].x ;
			int otherY = others[i].y ;
			if (others[i] != this) bounce(otherX, otherY) ;
		}
    }

	/** Update velocities if we touch another picture.		*/
    public void bounce(int nextX, int nextY) {

		int dx = x - nextX ;
		int dy = y - nextY;
		if ((Math.abs(dx) <= radius) && (Math.abs(dy) <= radius)) {
			velX = (int) (0.5 * dx);
			velY = (int) (0.5 * dy);
		}
	}

	/** Called by the glass when it changes position.		*/
    public void update(Observable foo, Object data) {

		Point theGlass = (Point) data ;
		int dx = x - theGlass.x ;
		int dy = y - theGlass.y ;

		// If the glass is over a picture we cover it
        if ( (Math.abs(dx) <= radius) && (Math.abs(dy) <= radius) ) {
			this.focus = theGlass ;
			((Advanced10) foo).setTarget(index) ;
		}
		// Otherwise we release the picture from the glass
		else {
			this.focus = null ;
			int old = ((Advanced10) foo).getTarget() ;
			if (old == index) ((Advanced10) foo).setTarget(0) ;
		}
    }

    /** Used to move the glass onto anoter location.	*/
    public void moveTo(int x, int y) {

        this.x = x;
        this.y = y;
		setChanged();
		notifyObservers( getPosition() );
    }

	/** Set the current target covered by the glass. 			*/
	public void setTarget(int n)	{	this.target = n;		}

	/** Return the current target covered by the glass. 		*/
	public int getTarget() 			{	return target; 			}

    /** Return the current position of this picture. 			*/
    public Point getPosition() 		{	return new Point(x, y);	}


}	// End of class