 /**
 * Title: Advanced11: a sample application showing binary operations
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.util.*;

/** The class managing input and output of values.	 */
class Advanced11 {

	int maxBits = 32 ;		// int are 4 bytes long

	/**	The Main method. */
    public static void main(String args[])    {

        System.out.println();
        Advanced11 theApp = new Advanced11();
        theApp.perform() ;
    }

    /**	Reads input from the user and show results on the screen. */
    public void perform()    {

		System.out.print("Enter first number : ") ;
		int a = getNumber() ;
		System.out.print("Enter second number: ") ;
		int b = getNumber() ;

		BitSet first = convert(a) ;
		BitSet second = convert(b) ;
		System.out.println("\nFirst : " + stringValue(first)) ;
		System.out.println("Second: " + stringValue(second) + "\n") ;

		// Summing the two numbers
		boolean carry = false ;
		BitSet sum = new BitSet(maxBits) ;
		BitAdder adder = new BitAdder() ;		// See below (helper class)
		for (int i = 0; i < maxBits; i++) {
			adder.sum(first.get(i), second.get(i), carry) ;
			if (adder.getSum()) sum.set(i) ;
			carry = adder.getCarry() ;
		}
		int result = intValue(sum) ;
		System.out.print("Result: " + stringValue(sum) + " (" + result + ")\n\n") ;

		// Now calculating the binary complement
		int complement = (~result) + 1 ;
		BitSet reverse = convert(complement) ;
		System.out.println("Compl.: " + stringValue(reverse) + " (" + complement + ")") ;

		// Finally we test the SIGNED right shift (2 positions)
		// REMARK: shifting of "n" position means dividing by 2^n 	!!!
		int shifted = (complement >> 2) ;
		BitSet smaller = convert(shifted) ;
		System.out.println("Shift : " + stringValue(smaller) + " (" + shifted + ")\n") ;
	}

    /**	Convert an <CODE>int</CODE> type into a <CODE>BitSet</CODE>.*/
    public BitSet convert(int number)    {

		int potence = 1;
		BitSet binary = new BitSet(maxBits) ;

		for (int i = 0; i < maxBits; i++) {
			int bit = (number & potence) ;		// Bitwise operation
			if (bit != 0) binary.set(i) ;
			potence *= 2 ;
		}

		return binary ;
	}

    /**	Convert an <CODE>BitSet</CODE> into a <CODE>String</CODE>.*/
    public String stringValue(BitSet binary)    {

		String result = "" ;
		for (int i = (maxBits - 1); i >= 0; i--) {
			if (binary.get(i)) result += "1" ;
				else result += "0" ;
		}

		return result ;
	}

    /**	Convert an <CODE>BitSet</CODE> into an <CODE>int</CODE> type.*/
    public int intValue(BitSet binary)    {

		int result = 0 ;
		int potence = 1;
		for (int i = 0; i < maxBits; i++) {
			if (binary.get(i)) result += potence;
			potence *= 2 ;
		}

		return result ;
	}

// ****************************			UTILITIES METHODS			***************************************

	/** Returns a numeric value from standard input. 		*/
    public int getNumber()    {

		int n = 0 ;
		int output = 99 ;		// Default value if an exception occurs
		String input = "" ;
        try {
			while (n != 13) {
				n = System.in.read() ;
				if (n != 13) input += (char) n ;
			}
			System.in.skip(1) ;
			output = Integer.parseInt(input) ;
		} catch(Exception e) { System.err.println("Wrong input: type only numbers, please ..."); }

		return output ;
	}

}	// End of class

// ****************************			HELPER CLASS			***************************************

/** The class used to sum one bit to another bit.								*/
class BitAdder {

	boolean sum, carry ;

	/** Sum two bit values considering an optional carry (from previous sum).	*/
    public void sum(boolean a, boolean b, boolean carry)    {

		boolean sum1 	= (a ^ carry) ;		// Byte "01" + byte "01" gives "10"
		boolean carry1 	= (a & carry) ;		// The carry will be added to the next bit
		this.sum		= (sum1 ^ b) ;
		boolean carry2 	= (sum1 & b) ;
		this.carry 		= (carry1 | carry2) ;
	}

	/** Return the last sum result.						*/
    public boolean getSum()    {	return this.sum ;	}

	/** Return the carry of the last sum.				*/
    public boolean getCarry()  {	return this.carry ;	}

}	// End of helper class