/**
 * Title: an example about URL and URLConnection
 * Description:
 * Copyright:    Copyright (c) 2004
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.advanced ;

import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

/** A text based simple browser.	 */
public class Advanced13 extends Frame implements ActionListener, WindowListener	{

	int index ;						// The index of the history element
	Vector history = new Vector();	// The history of typed URL's
	Parser parser ;					// The URL content parser (helper class)

	TextArea  document = new TextArea();
    TextField location = new TextField("", 42);
    Button    nextButton = new Button("Next");
    Button    homeButton = new Button("Home");
    Button    prevButton = new Button("Prev");
	Button    getButton  = new Button("Open");
	Button    showButton = new Button("Show");
    Button    quitButton = new Button("Quit");

	/** The Main method: just cretes the application.	 */
    public static void main(String args[]) {

        Advanced13 theApp = new Advanced13();
    }

	/** Standard connectionstructor: initializes the objects on the Frame.	 */
    public Advanced13()    {

        setTitle( "A very simple Browser" );
        location.setFont(new Font("Verdana", Font.PLAIN, 12));

		// Creating panels
        Panel p1 = new Panel();
        Panel p2 = new Panel();
        p2.setLayout( new GridLayout(10,1) ) ;
        p1.add(new Label("Enter a URL: "));
        p1.add(location);
        p2.add(homeButton);
        p2.add(getButton);
        p2.add(showButton);
        p2.add(prevButton);
        p2.add(nextButton);
        p2.add(quitButton);
        for (int i = 0 ; i < 3; i++) p2.add(new Label(" "));	// Formatting
        add(p1, "North");
        add(p2, "West");
        add(document, "Center");

		// Setting listeners
        location.addActionListener(this);
        prevButton.addActionListener(this);
        nextButton.addActionListener(this);
        homeButton.addActionListener(this);
		getButton.addActionListener(this);
		showButton.addActionListener(this);
        quitButton.addActionListener(this);
        addWindowListener(this);

        setSize(600, 500);
        setVisible(true);
        location.requestFocus();
        this.parser = new Parser(this) ;

        // Creating a very short history
        int index = -1 ;
        history.add(new String("http://www.ragnatele.it")) ;
        history.add(new String("http://www.libero.it")) ;
    }

	/** Called when one of the buttons is pressed. */
    public void actionPerformed(ActionEvent event) {

        if ( event.getSource() == getButton || event.getSource() == location ) {

            try {
                // Creates the URL object
                String typed = location.getText() ;
                addUrl(typed) ;
                URL url = new URL(typed) ;
                document.setText( "\nGetting: " + url + '\n' );

                // Retrieving information about the URL address
                String 	host = url.getHost();
                String 	page = url.getFile();
                String 	anchor = url.getRef();
				int 	port = url.getPort();
				String 	protocol = url.getProtocol();

                // Display URL information
                append("Host part of the URL : " + host 	+ '\n');
                append("File name of the URL : " + page		+ '\n');
                append("Ref part of the URL  : " + anchor	+ '\n');
                append("URL port number      : " + port		+ '\n');
				append("Protocol             : " + protocol	+ "\n\n");

                // Retrieving information about the connection
                URLConnection connection = url.openConnection();
                String  encoding = connection.getContentEncoding();
                int     length   = connection.getContentLength();
                String  type     = connection.getContentType();
                long    expires  = connection.getExpiration();
                Date    modified = new Date(connection.getLastModified());

                // Display document information
                append("Content encoding type: " + encoding + '\n');
                append("Content length       : " + length   + '\n');
                append("Content type         : " + type     + '\n');
				append("Last modified (date) : " + modified + '\n');
                append("Expires (date)       : " + expires  + '\n');

                // Display content type
                Object urlContent = url.getContent();
                String objectType = urlContent.getClass().getName();
                append("\nObject type: " + objectType + "\n\n");

                // Getting headers (if any)
                int n = 0 ;
                append("HEADER\n\n");
                String key = connection.getHeaderFieldKey(n) ;
                while (key != null) {
					append("Key: " + key + "   ");
					append("Value: " + connection.getHeaderField(n++) + '\n');
					key = connection.getHeaderFieldKey(n) ;
				}
            }

            // Handling Exceptions
            catch (MalformedURLException muException) {
                append("\nPlease, check your URL address\n" + muException);
            }
            catch (IOException ioException) {
                append("Sorry, not possible to read the URL content\n" + ioException);
            }
            catch (NullPointerException e3) {
                append("Sorry, URL content not found\n");
            }
        }
        else if (event.getSource( ) == nextButton) {
			if (index < (history.size()-1)) index++ ;
			setUrl() ;
		} else if (event.getSource( ) == prevButton) {
			if (index > 0) index-- ;
			setUrl() ;
		}
		else if (event.getSource( ) == showButton) parser.getContent(location.getText());
		else if (event.getSource( ) == homeButton) document.setText("");
        else if (event.getSource( ) == quitButton) quitApplication();
    }

	/** Add an address to the history. */
    public void addUrl(String s) {

		history.add(s) ;
		index = history.size() - 1 ;
	}

	/** Loads an address from the history. */
    public void setUrl() {

		String s = (String) history.elementAt(index) ;
		location.setText(s) ;
	}

	/** Called by the parser to add the output. 			*/
    public void append(String s) {	document.append(s) ;	}

    public void windowClosed      (WindowEvent event) {  }
    public void windowClosing     (WindowEvent event) { quitApplication( ); }
    public void windowIconified   (WindowEvent event) {  }
    public void windowDeiconified (WindowEvent event) {  }
    public void windowDeiconnectionified(WindowEvent event) { }
    public void windowIconnectionified  (WindowEvent event) { }
    public void windowOpened      (WindowEvent event) { }
    public void windowActivated   (WindowEvent event) { }
    public void windowDeactivated (WindowEvent event) { }

	/** Called when the window is closed or "Quit" is pressed. */
    public void quitApplication( ) {

        setVisible(false);
        dispose();
        System.exit(0);
    }

}	// End of class


// ********************************		HELPER CLASS 	****************************

/** A text based simple browser, offering two different ways of fetching data:<BR>
 *  1. simple:		reading from the InputStream "byte by byte".<BR>
 *  2. advanced:	reading from the InputStream using blocks "100 bytes long".<BR>
 */
class Parser {

	boolean simple = false ;	// We have two ways to read from InputStrem

	// CLASS ATTRIBUTES
	Advanced13 parent ;

	/** Standard constructor: set a  reference to the parent.	 */
	public Parser(Advanced13 parent) {

		this.parent = parent ;
	}

	/** Access the content from the URL. */
    public void getContent(String address) {

		try {
			URL url = new URL(address) ;
			Object urlContent = url.getContent();

			if (urlContent instanceof String) getString(urlContent) ;
			else if (urlContent instanceof InputStream) getStream(urlContent) ;
			else parent.append("Sorry, cannot show that type of object");
		}

		// Handling Exceptions
        catch (MalformedURLException muException) {
			parent.append("\nPlease, check your URL address\n" + muException);
		}
		catch (IOException ioException) {
			parent.append("Sorry, not possible to read the URL content\n" + ioException);
		}
		catch (Exception exception) {
			parent.append("Error" + exception);
		}
	}

	/** Method used to read String contents.	 */
	public void getString(Object content) {

		String s = (String) content ;
		parent.append(s) ;
	}

	/** Method used to read the content byte by byte.	 */
	public void getStream(Object content) throws IOException {

		if (simple) {
			int b = 0;
			while ( (b =((InputStream) content).read()) != -1 ) {
				parent.append("" + (char) b) ;
			}
		} else getLines(content) ;
	}

	/** Method used to read the content "50 chars per block".	 */
	public void getLines(Object content) throws IOException {

		int size = 100 ;
		byte[] b = new byte[size];
		while ( ((InputStream) content).read(b, 0, size) != -1 ) {
			parent.append(new String(b)) ;
		}
	}

}	// End of helper class
